@@ -5,7 +5,7 @@ class _variables():
 		self._cursor=conn.cursor()
 		valstr="id INT"
 		for var in values:
-			valstr+=",\n	"+var+" TEXT"
+			valstr+=",\n	var"+var+" TEXT"
 		self._cursor.execute("""CREATE TABLE IF NOT EXISTS globals(
 		"""+valstr+"""
 		)""")
@@ -14,18 +14,18 @@ class _variables():
 		)""")
 		for var in values:
 			try:
-				self._cursor.execute(f"alter table globals add column '{var}' 'TEXT'")
+				self._cursor.execute(f"alter table globals add column 'var{var}' 'TEXT'")
 			except:
 				pass
 			try:
-				self._cursor.execute(f"alter table servers add column '{var}' 'TEXT'")
+				self._cursor.execute(f"alter table servers add column 'var{var}' 'TEXT'")
 			except:
 				pass
 	def get(self,thing,type,where='11'):
-		self._cursor.execute(f'SELECT {thing} FROM {type} WHERE {where[0]} LIKE {where[1]}')
+		self._cursor.execute(f"SELECT var{thing} FROM {type} WHERE {where[0]} LIKE '{where[1]}'")
 		return self._cursor.fetchone()[0]
 	def set(self,things,type,where="11"):
-		self._cursor.execute(f"UPDATE {type} SET {things[0]}={things[1]} WHERE {where[0]} == {where[1]}")
+		self._cursor.execute(f"UPDATE {type} SET var{things[0]}='{things[1]}' WHERE {where[0]} == {where[1]}")
 		self._conn.commit()
 
 class dbd_bot():
@@ -42,6 +42,11 @@ class dbd_bot():
 		except:
 			print("Нет модуля asyncio, начинаем установку!")
 			os.system("pip install asyncio")
+		try:
+			import ast
+		except:
+			print("Нет модуля ast, начинаем установку!")
+			os.system("pip install ast")
 		import discord
 		from discord.ext import commands
 		self._prefix=info["prefix"]
@@ -52,6 +57,8 @@ class dbd_bot():
 		self._cooldown={}
 		self._cmds=[]
 		self._cs=0
+		self._func=""
+		self._functions={}
 		self._vars={}
 		self._nowcmd=""
 		try:
@@ -118,7 +125,14 @@ class dbd_bot():
 
 	def var(self, info):
 		self._vars[info["name"]]=info["value"]
-
+	
+	def pycode(self, info):
+		code=[]
+		for n in info:
+			if not n in ["name"]:
+				code.append(info[n])
+		self._functions[info["name"]]=code
+	
 	def run(self):
 		for c in self._codes["command"]:
 			self._cmds.append(self._codes["command"][c]["name"])
@@ -127,8 +141,9 @@ class dbd_bot():
 		_start(self._client, self)
 
 def _start(client, bot):
-	import time
 	import re
+	import ast
+	import time
 	import asyncio
 	import discord
 	import random
@@ -390,6 +405,26 @@ def _start(client, bot):
 					line=line.replace("$getReactions"+bot._style[0]+line.split("$getReactions"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],userstr)
 				except Exception as e:
 					line=line.replace("$getReactions"+bot._style[0]+line.split("$getReactions"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
+		while "$userDate"+bot._style[0] in line:
+			if nowloop==line:
+				break
+			else:
+				nowloop=line
+				try:
+					user = discord.utils.get(guild.members, id=int(line.split("$userDate"+bot._style[0])[-1].split(bot._style[1])[0]))
+					line=line.replace("$userDate"+bot._style[0]+line.split("$userDate"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],str(user.created_at))
+				except Exception as e:
+					line=line.replace("$userDate"+bot._style[0]+line.split("$userDate"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
+		while "$serverDate"+bot._style[0] in line:
+			if nowloop==line:
+				break
+			else:
+				nowloop=line
+				try:
+					server = discord.utils.get(client.guilds, id=int(line.split("$serverDate"+bot._style[0])[-1].split(bot._style[1])[0]))
+					line=line.replace("$serverDate"+bot._style[0]+line.split("$serverDate"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],str(server.created_at))
+				except Exception as e:
+					line=line.replace("$serverDate"+bot._style[0]+line.split("$serverDate"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
 		while "$randomText"+bot._style[0] in line:
 			if nowloop==line:
 				break
@@ -505,6 +540,15 @@ def _start(client, bot):
 				except Exception as e:
 					line=line.replace("$isNumber"+bot._style[0]+line.split("$isNumber"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
 		line=get_vars(line, guild, vars, author)
+		while "$kot"+bot._style[0] in line:
+			if nowloop==line:
+				break
+			else:
+				nowloop=line
+				try:
+					line=line.replace("$kot"+bot._style[0]+line.split("$kot"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],str(line.split("$kot"+bot._style[0])[-1].split(bot._style[1])[0][::-1]))
+				except Exception as e:
+					line=line.replace("$kot"+bot._style[0]+line.split("$kot"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
 		while "$replaceText"+bot._style[0] in line:
 			if nowloop==line:
 				break
@@ -610,26 +654,35 @@ def _start(client, bot):
 					tops={}
 					topstr=""
 					for i, u in top:
-						tops[str(i)]=u
+						try:
+							tops[str(i)]=int(u)
+						except Exception as e:
+							tops[str(i)]=u
 					top=sorted(tops.items(), key=lambda x: x[1], reverse=True)
-					for t, u in top:
+					for n, u in top:
 						try:
 							if topos<int(line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[2]) and int(line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[2])>=0:
 								if line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]=="server":
-									t = discord.utils.get(client.guilds, id=int(t))
+									t = discord.utils.get(client.guilds, id=int(n))
 								elif line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]=="global":
-									t = discord.utils.get(client.get_all_members(), id=int(t))
+									t = discord.utils.get(client.get_all_members(), id=int(n))
 								if t:
 									topos+=1
-									topstr+=str(topos)+". "+str(t)+" - "+str(u)+"\n"
+									try:
+										topstr+=str(line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[3]).replace("$pos",str(topos)).replace("$name",str(t.name)).replace("$id",str(t.id)).replace("$value",str(u)).replace("$full",str(t)).replace("$discr",str(t.discriminator))+"\n"
+									except:
+										topstr+=str(topos)+". "+str(t)+" - "+str(u)+"\n"
 							elif int(line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[2])==-1:
 								if line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]=="server":
-									t = discord.utils.get(client.guilds, id=int(t))
+									t = discord.utils.get(client.guilds, id=int(n))
 								elif line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]=="global":
-									t = discord.utils.get(client.get_all_members(), id=int(t))
+									t = discord.utils.get(client.get_all_members(), id=int(n))
 								if t:
 									topos+=1
-									topstr+=str(topos)+". "+str(t)+" - "+str(u)+"\n"
+									try:
+										topstr+=str(line.split("$leaderboard"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[3]).replace("$pos",str(topos)).replace("$name",str(t.name)).replace("$id",str(t.id)).replace("$value",str(u)).replace("$full",str(t)).replace("$discr",str(t.discriminator))+"\n"
+									except:
+										topstr+=str(topos)+". "+str(t)+" - "+str(u)+"\n"
 							else:
 								break
 						except:
@@ -1245,8 +1298,32 @@ def _start(client, bot):
 							restart=True
 							break
 						else:
-							if bot._debug:
-								print(f"Неизвестная функция {beforeline}")
+							def remember(text):
+								bot._func=text
+								return True
+							if any(line.startswith(func) and remember(func) for func in bot._functions):
+								def insert_returns(body):
+									if isinstance(body[-1], ast.Expr):
+										body[-1] = ast.Return(body[-1].value)
+										ast.fix_missing_locations(body[-1])
+										if isinstance(body[-1], ast.If):
+											insert_returns(body[-1].body)
+											insert_returns(body[-1].orelse)
+										if isinstance(body[-1], ast.With):
+											insert_returns(body[-1].body)
+									return body
+								fn_name = "_eval_expr"
+								cmd = "\n	".join(bot._functions[bot._func])
+								body = f"async def {fn_name}():\n	{cmd}"
+								parsed = ast.parse(body)
+								body = parsed.body[0].body
+								body = insert_returns(body)
+								env = { "arg":content, "message":message, "bot":client, 'discord': discord, 'commands': commands, 'ctx': ctx, '__import__': __import__ }
+								exec(compile(parsed, filename="<ast>", mode="exec"), env)
+								result = (await eval(f"{fn_name}()", env))
+							else:
+								if bot._debug:
+									print(f"Неизвестная функция {beforeline}")
 					except Exception as e:
 						if bot._debug:
 							print(f"Ошибка в функции {beforeline}")
@@ -1271,8 +1348,8 @@ def _start(client, bot):
 				pass
 			for var in bot._vars:
 				try:
-					if bot._db._cursor.execute(f"SELECT {var} FROM servers WHERE id = '{guild.id}'").fetchone()[0] is None:
-						bot._db._cursor.execute(f'UPDATE servers SET {var}="{bot._vars[var]}" WHERE id="{guild.id}"')
+					if bot._db._cursor.execute(f"SELECT var{var} FROM servers WHERE id = '{guild.id}'").fetchone()[0] is None:
+						bot._db._cursor.execute(f'UPDATE servers SET var{var}="{bot._vars[var]}" WHERE id="{guild.id}"')
 				except Exception as e:
 					pass
 			for member in guild.members:
@@ -1284,8 +1361,8 @@ def _start(client, bot):
 					pass
 				for var in bot._vars:
 					try:
-						if bot._db._cursor.execute(f"SELECT {var} FROM globals WHERE id = '{member.id}'").fetchone()[0] is None:
-							bot._db._cursor.execute(f'UPDATE globals SET {var}="{bot._vars[var]}" WHERE id="{member.id}"')
+						if bot._db._cursor.execute(f"SELECT var{var} FROM globals WHERE id = '{member.id}'").fetchone()[0] is None:
+							bot._db._cursor.execute(f'UPDATE globals SET var{var}="{bot._vars[var]}" WHERE id="{member.id}"')
 					except Exception as e:
 						pass
 		bot._db._conn.commit()
@@ -1297,14 +1374,20 @@ def _start(client, bot):
 	@client.event
 	async def on_member_join(user):
 		try:
-			await use(user, "onJoined", "", "event")
+			await use(user, "onJoin", "", "event")
+		except Exception as e:
+			pass
+	@client.event
+	async def on_message_delete(message):
+		try:
+			await use(message, "messageDelete", "", "event")
 		except Exception as e:
 			pass
 	
 	@client.event
 	async def on_guild_join(guild):
 		try:
-			await use(guild, "onBotJoin", "", "event")
+			await use(guild, "botJoin", "", "event")
 		except Exception as e:
 			pass
 	
